
/* GENERATED CODE... DO NOT EDIT */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 *
 * Copyright (C) 2013 - 2018 Aleksander Morgado <aleksander@aleksander.es>
 */


#include <string.h>

#include "mbim-dss.h"
#include "mbim-message-private.h"
#include "mbim-enum-types.h"
#include "mbim-error-types.h"
#include "mbim-device.h"
#include "mbim-utils.h"

/*****************************************************************************/
/* Message (Set): MBIM Message DSS Connect */

/**
 * mbim_message_dss_connect_set_new:
 * @device_service_id: the 'DeviceServiceId' field, given as a #MbimUuid.
 * @dss_session_id: the 'DssSessionId' field, given as a #guint32.
 * @dss_link_state: the 'DssLinkState' field, given as a #MbimDssLinkState.
 * @error: return location for error or %NULL.
 *
 * Create a new request for the 'Connect' set command in the 'DSS' service.
 *
 * Returns: a newly allocated #MbimMessage, which should be freed with mbim_message_unref().
 */
MbimMessage *
mbim_message_dss_connect_set_new (
    const MbimUuid *device_service_id,
    guint32 dss_session_id,
    MbimDssLinkState dss_link_state,
    GError **error)
{
    MbimMessageCommandBuilder *builder;

    builder = _mbim_message_command_builder_new (0,
                                                 MBIM_SERVICE_DSS,
                                                 MBIM_CID_DSS_CONNECT,
                                                 MBIM_MESSAGE_COMMAND_TYPE_SET);
    {
        _mbim_message_command_builder_append_uuid (builder, device_service_id);
    }
    {
        _mbim_message_command_builder_append_guint32 (builder, dss_session_id);
    }
    {
        _mbim_message_command_builder_append_guint32 (builder, dss_link_state);
    }

    return _mbim_message_command_builder_complete (builder);
}

static gchar *
mbim_message_dss_connect_set_get_printable (
    const MbimMessage *message,
    const gchar *line_prefix,
    GError **error)
{
    GString *str;
    guint32 offset = 0;

    str = g_string_new ("");

    g_string_append_printf (str, "%s  DeviceServiceId = ", line_prefix);
    {
        const MbimUuid *tmp;
        gchar *tmpstr;

        tmp = _mbim_message_read_uuid (message, offset);
        offset += 16;
        tmpstr = mbim_uuid_get_printable (tmp);
        g_string_append_printf (str, "'%s'", tmpstr);
        g_free (tmpstr);
    }
    g_string_append (str, "\n");

    g_string_append_printf (str, "%s  DssSessionId = ", line_prefix);
    {
        guint32 tmp;

        tmp = (guint32) _mbim_message_read_guint32 (message, offset);
        offset += 4;
        g_string_append_printf (str, "'%" G_GUINT32_FORMAT "'", tmp);
    }
    g_string_append (str, "\n");

    g_string_append_printf (str, "%s  DssLinkState = ", line_prefix);
    {
        MbimDssLinkState tmp;

        tmp = (MbimDssLinkState) _mbim_message_read_guint32 (message, offset);
        offset += 4;
#if defined __MBIM_DSS_LINK_STATE_IS_ENUM__
        g_string_append_printf (str, "'%s'", mbim_dss_link_state_get_string (tmp));
#elif defined __MBIM_DSS_LINK_STATE_IS_FLAGS__
        {
            gchar *tmpstr;

            tmpstr = mbim_dss_link_state_build_string_from_mask (tmp);
            g_string_append_printf (str, "'%s'", tmpstr);
            g_free (tmpstr);
        }
#else
# error neither enum nor flags
#endif

    }
    g_string_append (str, "\n");

    return g_string_free (str, FALSE);
}

/*****************************************************************************/
/* Message (Response): MBIM Message DSS Connect */

/**
 * mbim_message_dss_connect_response_parse:
 * @message: the #MbimMessage.
 * @error: return location for error or %NULL.
 *
 * Parses and returns parameters of the 'Connect' response command in the 'DSS' service.
 *
 * Returns: %TRUE if the message was correctly parsed, %FALSE if @error is set.
 */
gboolean
mbim_message_dss_connect_response_parse (
    const MbimMessage *message,
    GError **error)
{

    if (mbim_message_get_message_type (message) != MBIM_MESSAGE_TYPE_COMMAND_DONE) {
        g_set_error (error,
                     MBIM_CORE_ERROR,
                     MBIM_CORE_ERROR_INVALID_MESSAGE,
                     "Message is not a response");
        return FALSE;
    }

    return TRUE;
}

static gchar *
mbim_message_dss_connect_response_get_printable (
    const MbimMessage *message,
    const gchar *line_prefix,
    GError **error)
{
    GString *str;

    if (!mbim_message_response_get_result (message, MBIM_MESSAGE_TYPE_COMMAND_DONE, NULL))
        return NULL;

    str = g_string_new ("");

    return g_string_free (str, FALSE);
}

typedef struct {
  gchar * (* query_cb)        (const MbimMessage *message, const gchar *line_prefix, GError **error);
  gchar * (* set_cb)          (const MbimMessage *message, const gchar *line_prefix, GError **error);
  gchar * (* response_cb)     (const MbimMessage *message, const gchar *line_prefix, GError **error);
  gchar * (* notification_cb) (const MbimMessage *message, const gchar *line_prefix, GError **error);
} GetPrintableCallbacks;

static const GetPrintableCallbacks get_printable_callbacks[] = {
    [MBIM_CID_DSS_CONNECT] = {
        .set_cb = mbim_message_dss_connect_set_get_printable,
        .response_cb = mbim_message_dss_connect_response_get_printable,
    },
};

gchar *
__mbim_message_dss_get_printable_fields (
    const MbimMessage *message,
    const gchar *line_prefix,
    GError **error)
{
    guint32 cid;

    switch (mbim_message_get_message_type (message)) {
        case MBIM_MESSAGE_TYPE_COMMAND: {
            cid = mbim_message_command_get_cid (message);
            if (cid < G_N_ELEMENTS (get_printable_callbacks)) {
                switch (mbim_message_command_get_command_type (message)) {
                    case MBIM_MESSAGE_COMMAND_TYPE_QUERY:
                        if (get_printable_callbacks[cid].query_cb)
                            return get_printable_callbacks[cid].query_cb (message, line_prefix, error);
                        break;
                    case MBIM_MESSAGE_COMMAND_TYPE_SET:
                        if (get_printable_callbacks[cid].set_cb)
                            return get_printable_callbacks[cid].set_cb (message, line_prefix, error);
                        break;
                    default:
                        g_set_error (error,
                                     MBIM_CORE_ERROR,
                                     MBIM_CORE_ERROR_INVALID_MESSAGE,
                                     "Invalid command type");
                        return NULL;
                }
            }
            break;
        }

        case MBIM_MESSAGE_TYPE_COMMAND_DONE:
            cid = mbim_message_command_done_get_cid (message);
            if (cid < G_N_ELEMENTS (get_printable_callbacks)) {
                if (get_printable_callbacks[cid].response_cb)
                    return get_printable_callbacks[cid].response_cb (message, line_prefix, error);
            }
            break;

        case MBIM_MESSAGE_TYPE_INDICATE_STATUS:
            cid = mbim_message_indicate_status_get_cid (message);
            if (cid < G_N_ELEMENTS (get_printable_callbacks)) {
                if (get_printable_callbacks[cid].notification_cb)
                    return get_printable_callbacks[cid].notification_cb (message, line_prefix, error);
            }
            break;

        default:
            g_set_error (error,
                         MBIM_CORE_ERROR,
                         MBIM_CORE_ERROR_INVALID_MESSAGE,
                         "No contents expected in this message type");
            return NULL;
    }

    g_set_error (error,
                 MBIM_CORE_ERROR,
                 MBIM_CORE_ERROR_INVALID_MESSAGE,
                 "Unknown contents");
    return NULL;
}
